# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""This package includes the necessary classes for creating the main window for the
application."""

from .main_window import MainWindowFactory

__all__ = ["MainWindowFactory"]
