# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from .plugin_manager_dialog import PluginManagerDialog, PluginManagerDialogFactory

__all__ = ["PluginManagerDialog", "PluginManagerDialogFactory"]
